@@ -230,8 +230,8 @@ float pl_color_transfer_nominal_peak(enum pl_color_transfer trc)
     case PL_COLOR_TRC_GAMMA28:
     case PL_COLOR_TRC_PRO_PHOTO:
         return 1.0;
-    case PL_COLOR_TRC_PQ:       return 10000.0 / PL_COLOR_REF_WHITE;
-    case PL_COLOR_TRC_HLG:      return 12.0;
+    case PL_COLOR_TRC_PQ:       return 10000.0 / PL_COLOR_SDR_WHITE;
+    case PL_COLOR_TRC_HLG:      return 12.0 / PL_COLOR_SDR_WHITE_HLG;
     case PL_COLOR_TRC_V_LOG:    return 46.0855;
     case PL_COLOR_TRC_S_LOG1:   return 6.52;
     case PL_COLOR_TRC_S_LOG2:   return 9.212;
@@ -371,6 +371,9 @@ void pl_chroma_location_offset(enum pl_chroma_location loc, float *x, float *y)
 {
     *x = *y = 0;
 
+    // This is the majority of subsampled chroma content out there
+    loc = PL_DEF(loc, PL_CHROMA_LEFT);
+
     switch (loc) {
     case PL_CHROMA_LEFT:
     case PL_CHROMA_TOP_LEFT:
