@@ -65,7 +65,6 @@ struct pass_var {
 struct pass {
     uint64_t signature; // as returned by pl_shader_signature
     const struct pl_pass *pass;
-    bool failed;
 
     // contains cached data and update metadata, same order as pl_shader
     struct pass_var *vars;
@@ -303,12 +302,19 @@ static void generate_shaders(struct pl_dispatch *dp, struct pass *pass,
         ADD(pre, "#extension GL_OES_EGL_image_external : enable\n");
 
     if (gpu->glsl.gles) {
-        ADD(pre, "precision mediump float;\n");
-        ADD(pre, "precision mediump sampler2D;\n");
+        // Use 32-bit precision for floats if possible
+        ADD(pre, "#ifdef GL_FRAGMENT_PRECISION_HIGH \n"
+                 "precision highp float;            \n"
+                 "#else                             \n"
+                 "precision mediump float;          \n"
+                 "#endif                            \n");
+
+        // Always use 16-bit precision for samplers
+        ADD(pre, "precision mediump sampler2D; \n");
         if (gpu->limits.max_tex_1d_dim)
-            ADD(pre, "precision mediump sampler1D;\n");
+            ADD(pre, "precision mediump sampler1D; \n");
         if (gpu->limits.max_tex_3d_dim && gpu->glsl.version > 100)
-            ADD(pre, "precision mediump sampler3D;\n");
+            ADD(pre, "precision mediump sampler3D; \n");
     }
 
     char *vert_in  = gpu->glsl.version >= 130 ? "in" : "attribute";
@@ -589,17 +595,18 @@ static struct pass *find_pass(struct pl_dispatch *dp, struct pl_shader *sh,
 
     void *tmp = talloc_new(NULL); // for resources attached to `params`
 
-    struct pass *pass = talloc_zero(dp, struct pass);
-    pass->signature = sig;
-    pass->ubo_desc = (struct pl_shader_desc) {
-        .desc = {
-            .name = "UBO",
-            .type = PL_DESC_BUF_UNIFORM,
+    struct pass *pass = talloc_ptrtype(dp, pass);
+    *pass = (struct pass) {
+        .signature = sig,
+        .ubo_desc = {
+            .desc = {
+                .name = "UBO",
+                .type = PL_DESC_BUF_UNIFORM,
+            },
         },
     };
 
     struct pl_shader_res *res = &sh->res;
-
     struct pl_pass_run_params *rparams = &pass->run_params;
     struct pl_pass_params params = {
         .type = pl_shader_is_compute(sh) ? PL_PASS_COMPUTE : PL_PASS_RASTER,
@@ -1050,7 +1057,7 @@ bool pl_dispatch_compute(struct pl_dispatch *dp,
     struct pass *pass = find_pass(dp, sh, NULL, NULL, NULL, false);
 
     // Silently return on failed passes
-    if (pass->failed)
+    if (!pass->pass)
         goto error;
 
     struct pl_pass_run_params *rparams = &pass->run_params;
