@@ -83,17 +83,7 @@ void pl_gpu_print_info(const struct pl_gpu *gpu, enum pl_log_level lev)
     if (pl_gpu_supports_interop(gpu)) {
         PL_MSG(gpu, lev, "    External API interop:");
 
-        // Pretty-print the device UUID
-        static const char *hexdigits = "0123456789ABCDEF";
-        char buf[3 * sizeof(gpu->uuid)];
-        for (int i = 0; i < sizeof(gpu->uuid); i++) {
-            uint8_t x = gpu->uuid[i];
-            buf[3 * i + 0] = hexdigits[x >> 4];
-            buf[3 * i + 1] = hexdigits[x & 0xF];
-            buf[3 * i + 2] = i == sizeof(gpu->uuid) - 1 ? '\0' : ':';
-        }
-
-        PL_MSG(gpu, lev, "      UUID: %s", buf);
+        PL_MSG(gpu, lev, "      UUID: %s", PRINT_UUID(gpu->uuid));
         PL_MSG(gpu, lev, "      PCI: %04x:%02x:%02x:%x",
                gpu->pci.domain, gpu->pci.bus, gpu->pci.device, gpu->pci.function);
         PL_MSG(gpu, lev, "      buf export caps: 0x%x",
@@ -1043,6 +1033,7 @@ const struct pl_pass *pl_pass_create(const struct pl_gpu *gpu,
         require(target_fmt);
         require(target_fmt->caps & PL_FMT_CAP_RENDERABLE);
         require(!params->blend_params || target_fmt->caps & PL_FMT_CAP_BLENDABLE);
+        require(!params->blend_params || params->load_target);
         break;
     case PL_PASS_COMPUTE:
         require(gpu->caps & PL_GPU_CAP_COMPUTE);
@@ -1060,7 +1051,14 @@ const struct pl_pass *pl_pass_create(const struct pl_gpu *gpu,
     for (int i = 0; i < params->num_descriptors; i++) {
         struct pl_desc desc = params->descriptors[i];
         require(desc.name);
-        // TODO: enforce disjoint bindings if possible?
+
+        // enforce disjoint descriptor bindings for each namespace
+        int namespace = pl_desc_namespace(gpu, desc.type);
+        for (int j = i+1; j < params->num_descriptors; j++) {
+            struct pl_desc other = params->descriptors[j];
+            require(desc.binding != other.binding ||
+                    namespace != pl_desc_namespace(gpu, other.type));
+        }
     }
 
     require(params->push_constants_size <= gpu->limits.max_pushc_size);
@@ -1332,7 +1330,7 @@ bool pl_tex_download_pbo(const struct pl_gpu *gpu, struct pl_buf_pool *pbo,
 
     if (pl_buf_poll(gpu, buf, 0)) {
         PL_TRACE(gpu, "pl_tex_download without buffer: blocking (slow path)");
-        while (pl_buf_poll(gpu, buf, 1000000)) ; // 1 ms
+        while (pl_buf_poll(gpu, buf, UINT64_MAX)) ;
     }
 
     return pl_buf_read(gpu, buf, 0, params->ptr, bufparams.size);
@@ -1396,8 +1394,14 @@ bool pl_tex_upload_texel(const struct pl_gpu *gpu, struct pl_dispatch *dp,
     };
 
     GLSL("imageStore(%s, %s(pos), color);\n", img, coord_types[dims]);
-    int groups[3] = { groups_x, pl_rect_h(params->rc), pl_rect_d(params->rc) };
-    return pl_dispatch_compute(dp, &sh, groups);
+    return pl_dispatch_compute(dp, &(struct pl_dispatch_compute_params) {
+        .shader = &sh,
+        .dispatch_size = {
+            groups_x,
+            pl_rect_h(params->rc),
+            pl_rect_d(params->rc),
+        },
+    });
 
 error:
     return false;
@@ -1459,8 +1463,14 @@ bool pl_tex_download_texel(const struct pl_gpu *gpu, struct pl_dispatch *dp,
         GLSL("imageStore(%s, base + %d, vec4(color[%d])); \n", buf, i, i);
     GLSL("}\n");
 
-    int groups[3] = { groups_x, pl_rect_h(params->rc), pl_rect_d(params->rc) };
-    return pl_dispatch_compute(dp, &sh, groups);
+    return pl_dispatch_compute(dp, &(struct pl_dispatch_compute_params) {
+        .shader = &sh,
+        .dispatch_size = {
+            groups_x,
+            pl_rect_h(params->rc),
+            pl_rect_d(params->rc),
+        },
+    });
 
 error:
     return false;
@@ -1522,6 +1532,52 @@ void pl_sync_destroy(const struct pl_gpu *gpu,
 bool pl_tex_export(const struct pl_gpu *gpu, const struct pl_tex *tex,
                    const struct pl_sync *sync)
 {
+    require(tex->params.import_handle || tex->params.export_handle);
+
     const struct pl_gpu_fns *impl = TA_PRIV(gpu);
     return impl->tex_export(gpu, tex, sync);
+
+error:
+    return false;
+}
+
+struct pl_timer *pl_timer_create(const struct pl_gpu *gpu)
+{
+    const struct pl_gpu_fns *impl = TA_PRIV(gpu);
+    if (!impl->timer_create)
+        return NULL;
+
+    return impl->timer_create(gpu);
+}
+
+void pl_timer_destroy(const struct pl_gpu *gpu, struct pl_timer **timer)
+{
+    if (!*timer)
+        return;
+
+    const struct pl_gpu_fns *impl = TA_PRIV(gpu);
+    impl->timer_destroy(gpu, *timer);
+    *timer = NULL;
+}
+
+uint64_t pl_timer_query(const struct pl_gpu *gpu, struct pl_timer *timer)
+{
+    if (!timer)
+        return 0;
+
+    const struct pl_gpu_fns *impl = TA_PRIV(gpu);
+    return impl->timer_query(gpu, timer);
+}
+
+const char *print_uuid(char buf[3 * UUID_SIZE], const uint8_t uuid[UUID_SIZE])
+{
+    static const char *hexdigits = "0123456789ABCDEF";
+    for (int i = 0; i < UUID_SIZE; i++) {
+        uint8_t x = uuid[i];
+        buf[3 * i + 0] = hexdigits[x >> 4];
+        buf[3 * i + 1] = hexdigits[x & 0xF];
+        buf[3 * i + 2] = i == UUID_SIZE - 1 ? '\0' : ':';
+    }
+
+    return buf;
 }
