@@ -24,6 +24,7 @@
 #include <libplacebo/shaders/av1.h>
 #include <libplacebo/shaders/colorspace.h>
 #include <libplacebo/shaders/sampling.h>
+#include <libplacebo/shaders/custom.h>
 #include <libplacebo/swapchain.h>
 
 struct pl_renderer;
@@ -118,6 +119,11 @@ struct pl_render_params {
     // If NULL, this feature is disabled.
     const struct pl_cone_params *cone_params;
 
+    // List of custom user shaders / hooks.
+    // See <libplacebo/shaders/custom.h> for more information.
+    const struct pl_hook * const *hooks;
+    int num_hooks;
+
     // --- Performance / quality trade-off options:
     // These should generally be left off where quality is desired, as they can
     // degrade the result quite noticeably; but may be useful for older or
@@ -197,10 +203,22 @@ struct pl_render_params {
 };
 
 // This contains the default/recommended options for reasonable image quality,
-// while also not being too terribly slow. All of the *_params structs
-// are defaulted to the corresponding *_default_params.
+// while also not being too terribly slow. All of the *_params structs are
+// defaulted to the corresponding *_default_params, except for deband_params,
+// which is disabled by default.
+//
+// This should be fine on most integrated GPUs, but if it's too slow, consider
+// setting the params to {0} instead, or alternatively setting
+// `pl_render_params.disable_fbos` to true.
 extern const struct pl_render_params pl_render_default_params;
 
+// This contains a higher quality preset for better image quality at the cost
+// of quite a bit of performance. In addition to the settings implied by
+// `pl_render_default_params`, it sets the upscaler to `pl_filter_ewa_lanczos`,
+// and enables debanding. This should only really be used with a discrete GPU
+// and where maximum image quality is desired.
+extern const struct pl_render_params pl_render_high_quality_params;
+
 #define PL_MAX_PLANES 4
 
 // High level description of a single slice of an image. This basically
@@ -320,15 +338,6 @@ struct pl_image {
     // Optional ICC profile associated with this image.
     struct pl_icc_profile profile;
 
-    // The reference dimensions of this image. For typical content, this is the
-    // dimensions of the largest (non-subsampled) plane, e.g. luma. Note that
-    // for anamorphic content, this is the size of the texture itself, not the
-    // "nominal" size of the video. (Anamorphic pixel ratio conversions are
-    // done implicitly by differing the aspect ratio between `src_rect` and
-    // `dst_rect`)
-    int width;
-    int height;
-
     // The source rectangle which we want to render from, relative to the
     // reference dimensions. Pixels outside of this rectangle will ostensibly
     // be ignored, but note that they may still contribute to the output data
@@ -362,8 +371,17 @@ struct pl_image {
     // Associated AV1 grain params (see <libplacebo/shaders/av1.h>). This is
     // entirely optional, the default of {0} corresponds to no extra grain.
     struct pl_av1_grain_data av1_grain;
+
+    // Deprecated fields. These are no longer used and may safely be ignored.
+    int width, height;
 };
 
+// Helper function to infer the chroma location offset for each plane in an
+// image. This is equivalent to calling `pl_chroma_location_offset` on all
+// subsampled planes' shift_x/shift_y variables.
+void pl_image_set_chroma_location(struct pl_image *image,
+                                  enum pl_chroma_location chroma_loc);
+
 // Represents the target of a rendering operation
 struct pl_render_target {
     // The framebuffer (or texture) we want to render to. Must have `renderable`
@@ -375,7 +393,12 @@ struct pl_render_target {
     // larger or smaller than the src_rect, or if the aspect ratio is
     // different, scaling will occur. `dst_rect` may be flipped, and may be
     // partially or wholly outside the bounds of the fbo. (Optional)
-    struct pl_rect2d dst_rect;
+    //
+    // Note: Because the GPU can only render to a whole number of pixels, the
+    // actual rendered area is rounded to the nearest pixel. The only reason
+    // this is `pl_rect2df` and not `pl_rect2d` is to make it easier to
+    // directly use `pl_rect2df_aspect_*` functions on it.
+    struct pl_rect2df dst_rect;
 
     // The color representation and space of the output. If this does not match
     // the color space of the source, libplacebo will convert the colors
@@ -398,6 +421,11 @@ struct pl_render_target {
 void pl_render_target_from_swapchain(struct pl_render_target *out_target,
                                      const struct pl_swapchain_frame *frame);
 
+// Helper function to determine if the `target` covers the entire FBO or not.
+// If this returns true, users may want to `pl_tex_clear` the `target.fbo`
+// before calling `pl_render_image`.
+bool pl_render_target_partial(const struct pl_render_target *target);
+
 // Render a single image to a target using the given parameters. This is
 // fully dynamic, i.e. the params can change at any time. libplacebo will
 // internally detect and flush whatever caches are invalidated as a result of
