@@ -18,11 +18,6 @@
 #ifndef LIBPLACEBO_VULKAN_H_
 #define LIBPLACEBO_VULKAN_H_
 
-#include <libplacebo/config.h>
-#if !PL_HAVE_VULKAN
-#error Included <libplacebo/vulkan.h> but libplacebo built without vulkan!
-#endif
-
 #include <vulkan/vulkan.h>
 #include <libplacebo/gpu.h>
 #include <libplacebo/swapchain.h>
@@ -31,24 +26,37 @@
 struct pl_vk_inst {
     VkInstance instance;
 
+    // The Vulkan API version supported by this VkInstance.
+    uint32_t api_version;
+
     // The associated vkGetInstanceProcAddr pointer.
     PFN_vkGetInstanceProcAddr get_proc_addr;
 
     // The instance extensions that were successfully enabled, including
     // extensions enabled by libplacebo internally. May contain duplicates.
-    const char **extensions;
+    const char * const *extensions;
     int num_extensions;
 
     // The instance layers that were successfully enabled, including
     // layers enabled by libplacebo internally. May contain duplicates.
-    const char **layers;
+    const char * const *layers;
     int num_layers;
 };
 
 struct pl_vk_inst_params {
-    // If set, enable the debugging and validation layers.
+    // If set, enable the debugging and validation layers. These should
+    // generally be lightweight and relatively harmless to enable.
     bool debug;
 
+    // If set, also enable GPU-assisted verification and best practices
+    // layers. (Note: May cause substantial slowdown and/or result in lots of
+    // false positive spam)
+    bool debug_extra;
+
+    // If nonzero, restricts the Vulkan API version to be at most this. This
+    // is only really useful for explicitly testing backwards compatibility.
+    uint32_t max_api_version;
+
     // Pointer to a user-provided `vkGetInstanceProcAddr`. If this is NULL,
     // libplacebo will use the directly linked version (if available).
     PFN_vkGetInstanceProcAddr get_proc_addr;
@@ -56,12 +64,12 @@ struct pl_vk_inst_params {
     // Enables extra instance extensions. Instance creation will fail if these
     // extensions are not all supported. The user may use this to enable e.g.
     // windowing system integration.
-    const char **extensions;
+    const char * const *extensions;
     int num_extensions;
 
     // Enables extra optional instance extensions. These are opportunistically
     // enabled if supported by the device, but otherwise skipped.
-    const char **opt_extensions;
+    const char * const *opt_extensions;
     int num_opt_extensions;
 
     // Enables extra layers. Instance creation will fail if these layers are
@@ -70,12 +78,12 @@ struct pl_vk_inst_params {
     // NOTE: Layers needed for required/optional extensions are automatically
     // enabled. The user does not specifically need to enable layers related
     // to extension support.
-    const char **layers;
+    const char * const *layers;
     int num_layers;
 
     // Enables extra optional layers. These are opportunistically enabled if
     // supported by the platform, but otherwise skipped.
-    const char **opt_layers;
+    const char * const *opt_layers;
     int num_opt_layers;
 };
 
@@ -90,6 +98,11 @@ const struct pl_vk_inst *pl_vk_inst_create(struct pl_context *ctx,
 
 void pl_vk_inst_destroy(const struct pl_vk_inst **inst);
 
+struct pl_vulkan_queue {
+    uint32_t index; // Queue family index
+    uint32_t count; // Queue family count
+};
+
 // Structure representing the actual vulkan device and associated GPU instance
 struct pl_vulkan {
     const struct pl_gpu *gpu;
@@ -103,23 +116,35 @@ struct pl_vulkan {
     VkPhysicalDevice phys_device;
     VkDevice device;
 
+    // The Vulkan API version supported by this VkPhysicalDevice.
+    uint32_t api_version;
+
     // The device extensions that were successfully enabled, including
     // extensions enabled by libplacebo internally. May contain duplicates.
-    const char **extensions;
+    const char * const *extensions;
     int num_extensions;
 
-    // The list of enabled queue families and their queue counts. This may
-    // include secondary queue families providing compute or transfer
-    // capabilities.
+    // The device features that were enabled at device creation time.
+    const VkPhysicalDeviceFeatures2KHR *features;
+
+    // The explicit queue families we are using to provide a given capability,
+    // or {0} if no appropriate dedicated queue family exists for this
+    // operation type.
+    //
+    // It's guaranteed that `queue_graphics` is always set, but the existence
+    // of the other two is optional, and libplacebo will only set them if
+    // they are different from the graphics queue. Note that queue_compute
+    // and queue_transfer may refer to the same queue family index.
+    struct pl_vulkan_queue queue_graphics; // provides VK_QUEUE_GRAPHICS_BIT
+    struct pl_vulkan_queue queue_compute;  // provides VK_QUEUE_COMPUTE_BIT
+    struct pl_vulkan_queue queue_transfer; // provides VK_QUEUE_TRANSFER_BIT
+
+    // For convenience, these are the same enabled queue families and their
+    // queue counts in list form. This list does not contain duplicates.
     const struct pl_vulkan_queue *queues;
     int num_queues;
 };
 
-struct pl_vulkan_queue {
-    int index; // Queue family index
-    int count; // Queue family count
-};
-
 struct pl_vulkan_params {
     // The vulkan instance. Optional, if NULL then libplacebo will internally
     // create a VkInstance with the settings from `instance_params`.
@@ -158,6 +183,16 @@ struct pl_vulkan_params {
     // list of devices and their names are logged at level PL_LOG_INFO.
     const char *device_name;
 
+    // When choosing the device, only choose a device with this exact UUID.
+    // This overrides `allow_software` and `device_name`. No effect if `device`
+    // is set.
+    //
+    // Note: This relies on instance-level support for at least one of the
+    // VK_KHR_external_*_capabilities extensions (or vulkan 1.1). If this field
+    // is set when the instance does not support it, an error will be
+    // generated.
+    uint8_t device_uuid[16];
+
     // When choosing the device, controls whether or not to also allow software
     // GPUs. No effect if `device` or `device_name` are set.
     bool allow_software;
@@ -186,19 +221,30 @@ struct pl_vulkan_params {
     // Enables extra device extensions. Device creation will fail if these
     // extensions are not all supported. The user may use this to enable e.g.
     // interop extensions.
-    const char **extensions;
+    const char * const *extensions;
     int num_extensions;
 
     // Enables extra optional device extensions. These are opportunistically
     // enabled if supported by the device, but otherwise skipped.
-    const char **opt_extensions;
+    const char * const *opt_extensions;
     int num_opt_extensions;
 
+    // Optional extra features to enable at device creation time. These are
+    // opportunistically enabled if supported by the physical device, but
+    // otherwise kept disabled. Users may include extra extension-specific
+    // features in the pNext chain, however these *must* all be
+    // extension-specific structs, i.e. the use of "meta-structs" like
+    // VkPhysicalDeviceVulkan11Features is not allowed.
+    const VkPhysicalDeviceFeatures2KHR *features;
+
+    // --- Misc/debugging options
+
     // Restrict specific features to e.g. work around driver bugs, or simply
     // for testing purposes
     pl_gpu_caps blacklist_caps; // capabilities to be excluded
     int max_glsl_version;       // limit the maximum GLSL version
     bool disable_events;        // disables usage of VkEvent completely
+    uint32_t max_api_version;   // limit the maximum vulkan API version
 };
 
 // Default/recommended parameters. Should generally be safe and efficient.
@@ -217,16 +263,24 @@ const struct pl_vulkan *pl_vulkan_create(struct pl_context *ctx,
 // Note that all resources allocated from this vulkan object (e.g. via the
 // `vk->ra` or using `pl_vulkan_create_swapchain`) *must* be explicitly
 // destroyed by the user before calling this.
+//
+// Also note that this function will block until all in-flight GPU commands are
+// finished processing. You can avoid this by manually calling `pl_gpu_finish`
+// before `pl_vulkan_destroy`.
 void pl_vulkan_destroy(const struct pl_vulkan **vk);
 
 struct pl_vulkan_device_params {
     // The instance to use. Required!
+    //
+    // NOTE: The VkInstance provided by the user *MUST* be created with the
+    // `VK_KHR_get_physical_device_properties2` extension enabled!
     VkInstance instance;
 
     // Mirrored from `pl_vulkan_params`. All of these fields are optional.
     PFN_vkGetInstanceProcAddr get_proc_addr;
     VkSurfaceKHR surface;
     const char *device_name;
+    uint8_t device_uuid[16];
     bool allow_software;
 };
 
@@ -285,14 +339,103 @@ const struct pl_swapchain *pl_vulkan_create_swapchain(const struct pl_vulkan *vk
 // who have `params->allow_suboptimal` enabled.
 bool pl_vulkan_swapchain_suboptimal(const struct pl_swapchain *sw);
 
-// VkImage interop API
+// Vulkan interop API, for sharing a single VkDevice (and associated vulkan
+// resources) directly with the API user. The use of this API is a bit sketchy
+// and requires careful communication of Vulkan API state.
+
+struct pl_vulkan_import_params {
+    // The vulkan instance. Required.
+    //
+    // NOTE: The VkInstance provided by the user *MUST* be created with the
+    // `VK_KHR_get_physical_device_properties2` extension enabled!
+    VkInstance instance;
+
+    // Pointer to `vkGetInstanceProcAddr`. If this is NULL, libplacebo will
+    // use the directly linked version (if available).
+    PFN_vkGetInstanceProcAddr get_proc_addr;
+
+    // The physical device selected by the user. Required.
+    VkPhysicalDevice phys_device;
+
+    // The logical device created by the user. Required.
+    VkDevice device;
+
+    // --- Logical device parameters
+
+    // List of all device-level extensions that were enabled. (Instance-level
+    // extensions need not be re-specified here, since it's guaranteed that any
+    // instance-level extensions that device-level extensions depend on were
+    // enabled at the instance level)
+    const char * const *extensions;
+    int num_extensions;
+
+    // Enabled queue families. At least `queue_graphics` is required.
+    //
+    // It's okay for multiple queue families to be specified with the same
+    // index, e.g. in the event that a dedicated compute queue also happens to
+    // be the dedicated transfer queue.
+    //
+    // It's also okay to leave the queue struct as {0} in the event that no
+    // dedicated queue exists for a given operation type. libplacebo will
+    // automatically fall back to using e.g. the graphics queue instead.
+    struct pl_vulkan_queue queue_graphics; // must support VK_QUEUE_GRAPHICS_BIT
+    struct pl_vulkan_queue queue_compute;  // must support VK_QUEUE_COMPUTE_BIT
+    struct pl_vulkan_queue queue_transfer; // must support VK_QUEUE_TRANSFER_BIT
+
+    // Enabled VkPhysicalDeviceFeatures. May be left as NULL, in which case
+    // libplacebo will assume no extra device features were enabled.
+    const VkPhysicalDeviceFeatures2KHR *features;
+
+    // --- Misc/debugging options
+
+    // Restrict specific features to e.g. work around driver bugs, or simply
+    // for testing purposes. See `pl_vulkan_params` for a description of these.
+    pl_gpu_caps blacklist_caps;
+    int max_glsl_version;
+    bool disable_events;
+    uint32_t max_api_version;
+};
+
+// Import an existing VkDevice instead of creating a new one, and wrap it into
+// a `pl_vulkan` abstraction. It's safe to `pl_vulkan_destroy` this, which will
+// destroy application state related to libplacebo but leave the underlying
+// VkDevice intact.
+const struct pl_vulkan *pl_vulkan_import(struct pl_context *ctx,
+                                         const struct pl_vulkan_import_params *params);
+
+struct pl_vulkan_wrap_params {
+    // The image itself. It *must* be usable concurrently by all of the queue
+    // family indices listed in `pl_vulkan->queues`. Note that this requires
+    // the use of VK_SHARING_MODE_CONCURRENT if `pl_vulkan->num_queues` is
+    // greater than 1. If this is difficult to achieve for the user, then
+    // `async_transfer` / `async_compute` should be turned off, which
+    // guarantees the use of only one queue family.
+    VkImage image;
+
+    // The image's dimensions (unused dimensions must be 0)
+    int width;
+    int height;
+    int depth;
+
+    // The image's format. libplacebo will try to map this to an equivalent
+    // pl_fmt. If no compatible pl_fmt is found, wrapping will fail.
+    VkFormat format;
+
+    // The usage flags the image was created with. libplacebo will set the
+    // pl_tex capabilities to include whatever it can, as determined by the set
+    // of enabled usage flags.
+    VkImageUsageFlags usage;
+
+    // The desired sampling / address modes of the resulting `pl_tex`. Note
+    // that PL_TEX_SAMPLE_LINEAR requires that `format` be a linearly
+    // sampleable texture format.
+    enum pl_tex_sample_mode sample_mode;
+    enum pl_tex_address_mode address_mode;
+};
 
 // Wraps an external VkImage into a pl_tex abstraction. By default, the image
-// is considered "held" by the user and will not be touched or used by
-// libplacebo until released (see `pl_vulkan_release`). Releasing an image
-// makes libplacebo take ownership of the image until the user calls
-// `pl_vulkan_hold` on it again. During this time, the user should not use
-// the VkImage in any way.
+// is considered "held" by the user and must be released before calling any
+// pl_tex_* API calls on it (see `pl_vulkan_release`).
 //
 // This wrapper can be destroyed by simply calling `pl_tex_destroy` on it,
 // which will not destroy the underlying VkImage. If a pl_tex wrapper is
@@ -303,24 +446,27 @@ bool pl_vulkan_swapchain_suboptimal(const struct pl_swapchain *sw);
 // to wrap an image belonging to a different VkDevice than the one in use by
 // `gpu`.
 //
-// The VkImage must be usable concurrently by all of the queue family indices
-// listed in `pl_vulkan->queues`. Note that this requires the use of
-// VK_SHARING_MODE_CONCURRENT if `pl_vulkan->num_queues` is greater than 1. If
-// this is difficult to achieve for the user, then `async_transfer` /
-// `async_compute` should be turned off, which guarantees the use of only one
-// queue family.
-//
-// This function may fail, for example if the VkFormat specified does not
-// map to any corresponding `pl_fmt`.
+// This function may fail, in which case it returns NULL.
 const struct pl_tex *pl_vulkan_wrap(const struct pl_gpu *gpu,
-                                    VkImage image, int w, int h, int d,
-                                    VkFormat format, VkImageUsageFlags usage);
+                                    const struct pl_vulkan_wrap_params *params);
+
+// For purely informative reasons, this contains a list of extensions and
+// device features that libplacebo *can* make use of. These are all strictly
+// optional, but provide a hint to the API user as to what might be worth
+// enabling at device creation time.
+//
+// Note: This also includes physical device features provided by extensions.
+// They are all provided using extension-specific features structs, rather
+// than the more general purpose VkPhysicalDeviceVulkan11Features etc.
+extern const char * const pl_vulkan_recommended_extensions[];
+extern const int pl_vulkan_num_recommended_extensions;
+extern const VkPhysicalDeviceFeatures2KHR pl_vulkan_recommended_features;
 
 // Analogous to `pl_vulkan_wrap`, this function takes any `pl_tex` (including
-// one created by libplacebo) and unwraps it to expose the underlying VkImage
-// to the user. Unlike `pl_vulkan_wrap`, this `pl_tex` is *not* considered held
-// by default - the user must explicitly `pl_vulkan_hold` before accessing the
-// VkImage for the first time.
+// ones created by `pl_tex_create`) and unwraps it to expose the underlying
+// VkImage to the user. Unlike `pl_vulkan_wrap`, this `pl_tex` is *not*
+// considered held after calling this function - the user must explicitly
+// `pl_vulkan_hold` before accessing the VkImage.
 //
 // `out_format` and `out_flags` will be updated to hold the VkImage's
 // format and usage flags. (Optional)
@@ -328,7 +474,7 @@ VkImage pl_vulkan_unwrap(const struct pl_gpu *gpu, const struct pl_tex *tex,
                          VkFormat *out_format, VkImageUsageFlags *out_flags);
 
 // "Hold" a shared image. This will transition the image into the layout and
-// access mode specified by the user, and fire the given semaphore (required!)
+// access mode specified by the user, and fire the given semaphore (optional)
 // when this is done. This marks the image as held. Attempting to perform any
 // pl_tex_* operation (except pl_tex_destroy) on a held image is undefined
 // behavior.
@@ -338,6 +484,16 @@ bool pl_vulkan_hold(const struct pl_gpu *gpu, const struct pl_tex *tex,
                     VkImageLayout layout, VkAccessFlags access,
                     VkSemaphore sem_out);
 
+// This function is similar to `pl_vulkan_hold`, except that rather than
+// forcibly transitioning to a given layout, the user is instead informed about
+// the current layout and access and is in charge of transitioning it to their
+// own layout/access before using it. May be more convenient for some users.
+//
+// Returns whether successful.
+bool pl_vulkan_hold_raw(const struct pl_gpu *gpu, const struct pl_tex *tex,
+                        VkImageLayout *layout, VkAccessFlags *access,
+                        VkSemaphore sem_out);
+
 // "Release" a shared image, meaning it is no longer held. `layout` and
 // `access` describe the current state of the image at the point in time when
 // the user is releasing it. Performing any operation on the VkImage underlying
