@@ -57,11 +57,9 @@ void pl_shader_reset(struct pl_shader *sh, const struct pl_shader_params *params
         .mutable = true,
 
         // Preserve array allocations
-        .res = {
-            .variables      = sh->res.variables,
-            .descriptors    = sh->res.descriptors,
-            .vertex_attribs = sh->res.vertex_attribs,
-        },
+        .variables      = sh->variables,
+        .descriptors    = sh->descriptors,
+        .vertex_attribs = sh->vertex_attribs,
     };
 
     if (params)
@@ -108,6 +106,21 @@ bool sh_try_compute(struct pl_shader *sh, int bw, int bh, bool flex, size_t mem)
         return false;
     }
 
+    if (bw > gpu->limits.max_group_size[0] ||
+        bh > gpu->limits.max_group_size[1] ||
+        (bw * bh) > gpu->limits.max_group_threads)
+    {
+        if (!flex) {
+            PL_TRACE(sh, "Disabling compute shader due to exceeded group "
+                     "thread count.");
+            return false;
+        } else {
+            // Pick better group sizes
+            bw = PL_MIN(bw, gpu->limits.max_group_size[0]);
+            bh = gpu->limits.max_group_threads / bw;
+        }
+    }
+
     sh->res.compute_shmem += mem;
 
     // If the current shader is either not a compute shader, or we have no
@@ -123,6 +136,7 @@ bool sh_try_compute(struct pl_shader *sh, int bw, int bh, bool flex, size_t mem)
     if (sh->flexible_work_groups && flex) {
         *sh_bw = PL_MAX(*sh_bw, bw);
         *sh_bh = PL_MAX(*sh_bh, bh);
+        pl_assert(*sh_bw * *sh_bh <= gpu->limits.max_group_threads);
         return true;
     }
 
@@ -178,7 +192,7 @@ ident_t sh_var(struct pl_shader *sh, struct pl_shader_var sv)
 {
     sv.var.name = sh_fresh(sh, sv.var.name);
     sv.data = talloc_memdup(sh->tmp, sv.data, pl_var_host_layout(0, &sv.var).size);
-    TARRAY_APPEND(sh, sh->res.variables, sh->res.num_variables, sv);
+    TARRAY_APPEND(sh, sh->variables, sh->res.num_variables, sv);
     return (ident_t) sv.var.name;
 }
 
@@ -192,15 +206,15 @@ ident_t sh_desc(struct pl_shader *sh, struct pl_shader_desc sd)
     case PL_DESC_BUF_TEXEL_UNIFORM:
     case PL_DESC_BUF_TEXEL_STORAGE:
         for (int i = 0; i < sh->res.num_descriptors; i++) {
-            if (sh->res.descriptors[i].object == sd.object)
-                return (ident_t) sh->res.descriptors[i].desc.name;
+            if (sh->descriptors[i].object == sd.object)
+                return (ident_t) sh->descriptors[i].desc.name;
         }
 
     default: break;
     }
 
     sd.desc.name = sh_fresh(sh, sd.desc.name);
-    TARRAY_APPEND(sh, sh->res.descriptors, sh->res.num_descriptors, sd);
+    TARRAY_APPEND(sh, sh->descriptors, sh->res.num_descriptors, sd);
     return (ident_t) sd.desc.name;
 }
 
@@ -235,7 +249,7 @@ ident_t sh_attr_vec2(struct pl_shader *sh, const char *name,
         .data = { &data[0], &data[2], &data[4], &data[6] },
     };
 
-    TARRAY_APPEND(sh, sh->res.vertex_attribs, sh->res.num_vertex_attribs, va);
+    TARRAY_APPEND(sh, sh->vertex_attribs, sh->res.num_vertex_attribs, va);
     return (ident_t) va.attr.name;
 }
 
@@ -261,6 +275,15 @@ ident_t sh_bind(struct pl_shader *sh, const struct pl_tex *tex,
         .object = tex,
     });
 
+    float sx, sy;
+    if (tex->sampler_type == PL_SAMPLER_RECT) {
+        sx = 1.0;
+        sy = 1.0;
+    } else {
+        sx = 1.0 / tex->params.w;
+        sy = 1.0 / tex->params.h;
+    }
+
     if (out_pos) {
         struct pl_rect2df full = {
             .x1 = tex->params.w,
@@ -269,8 +292,8 @@ ident_t sh_bind(struct pl_shader *sh, const struct pl_tex *tex,
 
         rect = PL_DEF(rect, &full);
         *out_pos = sh_attr_vec2(sh, "tex_coord", &(struct pl_rect2df) {
-            .x0 = rect->x0 / tex->params.w, .y0 = rect->y0 / tex->params.h,
-            .x1 = rect->x1 / tex->params.w, .y1 = rect->y1 / tex->params.h,
+            .x0 = sx * rect->x0, .y0 = sy * rect->y0,
+            .x1 = sx * rect->x1, .y1 = sy * rect->y1,
         });
     }
 
@@ -284,7 +307,7 @@ ident_t sh_bind(struct pl_shader *sh, const struct pl_tex *tex,
     if (out_pt) {
         *out_pt = sh_var(sh, (struct pl_shader_var) {
             .var  = pl_var_vec2("tex_pt"),
-            .data = &(float[2]) {1.0 / tex->params.w, 1.0 / tex->params.h},
+            .data = &(float[2]) {sx, sy},
         });
     }
 
@@ -340,27 +363,41 @@ void pl_shader_append(struct pl_shader *sh, enum pl_shader_buf buf,
     va_end(ap);
 }
 
-static const char *outsigs[] = {
-    [PL_SHADER_SIG_NONE]  = "void",
-    [PL_SHADER_SIG_COLOR] = "vec4",
-};
+void pl_shader_append_bstr(struct pl_shader *sh, enum pl_shader_buf buf,
+                           struct bstr str)
+{
+    pl_assert(buf >= 0 && buf < SH_BUF_COUNT);
+    bstr_xappend(sh, &sh->buffers[buf], str);
+}
 
 static const char *insigs[] = {
     [PL_SHADER_SIG_NONE]  = "",
     [PL_SHADER_SIG_COLOR] = "vec4 color",
 };
 
+static const char *outsigs[] = {
+    [PL_SHADER_SIG_NONE]  = "void",
+    [PL_SHADER_SIG_COLOR] = "vec4",
+};
+
 static const char *retvals[] = {
     [PL_SHADER_SIG_NONE]  = "",
     [PL_SHADER_SIG_COLOR] = "return color;",
 };
 
+// libplacebo currently only allows 2D samplers for shader signatures
+static const char *samplers2D[] = {
+    [PL_SAMPLER_NORMAL]     = "sampler2D",
+    [PL_SAMPLER_RECT]       = "sampler2DRect",
+    [PL_SAMPLER_EXTERNAL]   = "samplerExternalOES",
+};
+
 ident_t sh_subpass(struct pl_shader *sh, const struct pl_shader *sub)
 {
     pl_assert(sh->mutable);
 
     if (SH_PARAMS(sh).id == SH_PARAMS(sub).id) {
-        SH_FAIL(sh, "Failed merging shaders: conflicting identifiers!");
+        PL_TRACE(sh, "Can't merge shaders: conflicting identifiers!");
         return NULL;
     }
 
@@ -371,8 +408,8 @@ ident_t sh_subpass(struct pl_shader *sh, const struct pl_shader *sub)
     if ((sub->output_w && res_w != sub->output_w) ||
         (sub->output_h && res_h != sub->output_h))
     {
-        SH_FAIL(sh, "Failed merging shaders: incompatible sizes: %dx%d and %dx%d",
-                sh->output_w, sh->output_h, sub->output_w, sub->output_h);
+        PL_TRACE(sh, "Can't merge shaders: incompatible sizes: %dx%d and %dx%d",
+                 sh->output_w, sh->output_h, sub->output_w, sub->output_h);
         return NULL;
     }
 
@@ -382,8 +419,8 @@ ident_t sh_subpass(struct pl_shader *sh, const struct pl_shader *sub)
         bool flex = sub->flexible_work_groups;
 
         if (!sh_try_compute(sh, subw, subh, flex, sub->res.compute_shmem)) {
-            SH_FAIL(sh, "Failed merging shaders: incompatible block sizes or "
-                    "exceeded shared memory resource capabilities");
+            PL_TRACE(sh, "Can't merge shaders: incompatible block sizes or "
+                     "exceeded shared memory resource capabilities");
             return NULL;
         }
     }
@@ -397,14 +434,20 @@ ident_t sh_subpass(struct pl_shader *sh, const struct pl_shader *sub)
 
     // Append the body as a new header function
     ident_t name = sh_fresh(sh, "sub");
-    GLSLH("%s %s(%s) {\n", outsigs[sub->res.output], name, insigs[sub->res.input]);
+    if (sub->res.input == PL_SHADER_SIG_SAMPLER) {
+        pl_assert(sub->sampler_prefix);
+        GLSLH("%s %s(%c%s src_tex, vec2 tex_coord) {\n",
+              outsigs[sub->res.output], name,
+              sub->sampler_prefix, samplers2D[sub->sampler_type]);
+    } else {
+        GLSLH("%s %s(%s) {\n", outsigs[sub->res.output], name, insigs[sub->res.input]);
+    }
     bstr_xappend(sh, &sh->buffers[SH_BUF_HEADER], sub->buffers[SH_BUF_BODY]);
     GLSLH("%s\n}\n\n", retvals[sub->res.output]);
 
     // Copy over all of the descriptors etc.
     talloc_ref_attach(sh->tmp, sub->tmp);
-#define COPY(f) TARRAY_CONCAT(sh, sh->res.f, sh->res.num_##f, \
-                              sub->res.f, sub->res.num_##f)
+#define COPY(f) TARRAY_CONCAT(sh, sh->f, sh->res.num_##f, sub->f, sub->res.num_##f)
     COPY(variables);
     COPY(descriptors);
     COPY(vertex_attribs);
@@ -420,7 +463,14 @@ static ident_t sh_split(struct pl_shader *sh)
 
     // Concatenate the body onto the head as a new function
     ident_t name = sh_fresh(sh, "main");
-    GLSLH("%s %s(%s) {\n", outsigs[sh->res.output], name, insigs[sh->res.input]);
+    if (sh->res.input == PL_SHADER_SIG_SAMPLER) {
+        pl_assert(sh->sampler_prefix);
+        GLSLH("%s %s(%c%s src_tex, vec2 tex_coord) {\n",
+              outsigs[sh->res.output], name,
+              sh->sampler_prefix, samplers2D[sh->sampler_type]);
+    } else {
+        GLSLH("%s %s(%s) {\n", outsigs[sh->res.output], name, insigs[sh->res.input]);
+    }
 
     if (sh->buffers[SH_BUF_BODY].len) {
         bstr_xappend(sh, &sh->buffers[SH_BUF_HEADER], sh->buffers[SH_BUF_BODY]);
@@ -458,6 +508,11 @@ const struct pl_shader_res *pl_shader_finalize(struct pl_shader *sh)
     struct bstr *glsl = &sh->buffers[SH_BUF_PRELUDE];
     bstr_xappend(sh, glsl, sh->buffers[SH_BUF_HEADER]);
 
+    // Set the vas/vars/descs
+    sh->res.vertex_attribs = sh->vertex_attribs;
+    sh->res.variables = sh->variables;
+    sh->res.descriptors = sh->descriptors;
+
     // Update the result pointer and return
     sh->res.glsl = glsl->start;
     sh->mutable = false;
@@ -467,7 +522,7 @@ const struct pl_shader_res *pl_shader_finalize(struct pl_shader *sh)
 bool sh_require(struct pl_shader *sh, enum pl_shader_sig insig, int w, int h)
 {
     if (sh->failed) {
-        PL_ERR(sh, "Attempting to modify a failed shader!");
+        SH_FAIL(sh, "Attempting to modify a failed shader!");
         return false;
     }
 
@@ -646,7 +701,7 @@ static void sh_lut_uninit(const struct pl_gpu *gpu, void *ptr)
 
 ident_t sh_lut(struct pl_shader *sh, struct pl_shader_obj **obj,
                enum sh_lut_method method, int width, int height, int depth,
-               int comps, bool update, void *priv,
+               int comps, bool update, bool dynamic, void *priv,
                void (*fill)(void *priv, float *data, int w, int h, int d))
 {
     const struct pl_gpu *gpu = SH_GPU(sh);
@@ -730,8 +785,6 @@ next_dim: ; // `continue` out of the inner loop
     }
 
     if (update) {
-        sh_lut_uninit(gpu, lut);
-
         tmp = talloc_zero_size(NULL, size * comps * sizeof(float));
         fill(priv, tmp, width, height, depth);
 
@@ -758,8 +811,7 @@ next_dim: ; // `continue` out of the inner loop
                 goto error;
             }
 
-            pl_assert(!lut->weights.tex);
-            lut->weights.tex = pl_tex_create(gpu, &(struct pl_tex_params) {
+            struct pl_tex_params params = {
                 .w              = width,
                 .h              = PL_DEF(height, texdim >= 2 ? 1 : 0),
                 .d              = PL_DEF(depth,  texdim >= 3 ? 1 : 0),
@@ -767,10 +819,26 @@ next_dim: ; // `continue` out of the inner loop
                 .sampleable     = true,
                 .sample_mode    = mode,
                 .address_mode   = PL_TEX_ADDRESS_CLAMP,
-                .initial_data   = tmp,
-            });
+                .host_writable  = dynamic,
+                .initial_data   = dynamic ? NULL : tmp,
+            };
 
-            if (!lut->weights.tex) {
+            bool ok;
+            if (dynamic) {
+                ok = pl_tex_recreate(gpu, &lut->weights.tex, &params);
+                if (ok) {
+                    ok = pl_tex_upload(gpu, &(struct pl_tex_transfer_params) {
+                        .tex = lut->weights.tex,
+                        .ptr = tmp,
+                    });
+                }
+            } else {
+                pl_tex_destroy(gpu, &lut->weights.tex);
+                lut->weights.tex = pl_tex_create(gpu, &params);
+                ok = lut->weights.tex;
+            }
+
+            if (!ok) {
                 SH_FAIL(sh, "Failed creating LUT texture!");
                 goto error;
             }
@@ -778,13 +846,13 @@ next_dim: ; // `continue` out of the inner loop
         }
 
         case SH_LUT_UNIFORM:
-            pl_assert(!lut->weights.data);
+            talloc_free(lut->weights.data);
             lut->weights.data = tmp; // re-use `tmp`
             tmp = NULL;
             break;
 
         case SH_LUT_LITERAL: {
-            pl_assert(!lut->weights.str.len);
+            lut->weights.str.len = 0;
             for (int i = 0; i < size * comps; i += comps) {
                 if (i > 0)
                     bstr_xappend_asprintf_c(lut, &lut->weights.str, ",");
@@ -857,7 +925,8 @@ next_dim: ; // `continue` out of the inner loop
             pos_macros[i] = sh_lut_pos(sh, sizes[i]);
 
         GLSLH("#define %s(pos) (%s(%s, %s(\\\n",
-              name, sh_tex_fn(sh, lut->weights.tex), tex, types[texdim - 1]);
+              name, sh_tex_fn(sh, lut->weights.tex->params),
+              tex, types[texdim - 1]);
 
         for (int i = 0; i < texdim; i++) {
             char sep = i == 0 ? ' ' : ',';
@@ -936,15 +1005,3 @@ const char *sh_bvec(const struct pl_shader *sh, int dims)
     pl_assert(dims > 0 && dims < PL_ARRAY_SIZE(bvecs));
     return sh_glsl(sh).version >= 130 ? bvecs[dims] : vecs[dims];
 }
-
-const char *sh_tex_fn(const struct pl_shader *sh, const struct pl_tex *tex)
-{
-    static const char *suffixed[] = {
-        [1] = "texture1D",
-        [2] = "texture2D",
-        [3] = "texture3D",
-    };
-
-    int dims = pl_tex_params_dimension(tex->params);
-    return sh_glsl(sh).version >= 130 ? "texture" : suffixed[dims];
-}
