@@ -37,41 +37,38 @@ enum pl_shader_buf {
 };
 
 struct pl_shader {
-    // Read-only fields
     struct pl_context *ctx;
-    const struct pl_gpu *gpu;
-
-    // Internal state
-    struct ta_ref *tmp;
+    struct pl_shader_res res; // for accumulating some of the fields
+    struct xta_ref *tmp; // only used for var/va/desc names and var/va data
     bool failed;
     bool mutable;
     int output_w;
     int output_h;
-    struct pl_shader_res res; // for accumulating vertex_attribs etc.
     struct bstr buffers[SH_BUF_COUNT];
     bool is_compute;
     bool flexible_work_groups;
-    uint8_t ident;
-    uint8_t index;
+    enum pl_sampler_type sampler_type;
+    char sampler_prefix;
     int fresh;
+
+    // mutable versions of the fields from pl_shader_res
+    struct pl_shader_va *vertex_attribs;
+    struct pl_shader_var *variables;
+    struct pl_shader_desc *descriptors;
 };
 
+// Helper functions for convenience
+#define SH_PARAMS(sh) ((sh)->res.params)
+#define SH_GPU(sh) (SH_PARAMS(sh).gpu)
+
+// Returns the GLSL description, defaulting to desktop 130.
+struct pl_glsl_desc sh_glsl(const struct pl_shader *sh);
+
 #define SH_FAIL(sh, ...) do {    \
         sh->failed = true;       \
         PL_ERR(sh, __VA_ARGS__); \
     } while (0)
 
-// Like `pl_shader_alloc`, but also has an extra `uint8_t ident` which can be
-// used to namespace shaders in order to allow safely merging together multiple
-// shaders using `sh_subpass`. This is not exposed publicly since there's no
-// reasonable public API for `sh_subpass`.
-struct pl_shader *pl_shader_alloc_ex(struct pl_context *ctx,
-                                     const struct pl_gpu *gpu,
-                                     uint8_t index, uint8_t ident);
-
-// Like `pl_shader_reset`, but also has this extra `uint8_t ident`.
-void pl_shader_reset_ex(struct pl_shader *sh, uint8_t index, uint8_t ident);
-
 // Attempt enabling compute shaders for this pass, if possible
 bool sh_try_compute(struct pl_shader *sh, int bw, int bh, bool flex, size_t mem);
 
@@ -126,6 +123,9 @@ void pl_shader_append(struct pl_shader *sh, enum pl_shader_buf buf,
                       const char *fmt, ...)
     PRINTF_ATTRIBUTE(3, 4);
 
+void pl_shader_append_bstr(struct pl_shader *sh, enum pl_shader_buf buf,
+                           struct bstr str);
+
 #define GLSLP(...) pl_shader_append(sh, SH_BUF_PRELUDE, __VA_ARGS__)
 #define GLSLH(...) pl_shader_append(sh, SH_BUF_HEADER, __VA_ARGS__)
 #define GLSL(...)  pl_shader_append(sh, SH_BUF_BODY, __VA_ARGS__)
@@ -196,12 +196,30 @@ enum sh_lut_method {
 // The `fill` function will be called with a zero-initialized buffer whenever
 // the data needs to be computed, which happens whenever the size is changed,
 // the shader object is invalidated, or `update` is set to true.
+//
+// If `dynamic` is set to true, shader objects will be preserved and updated
+// in-place rather than being treated as read-only.
 ident_t sh_lut(struct pl_shader *sh, struct pl_shader_obj **obj,
                enum sh_lut_method method, int width, int height, int depth,
-               int components, bool update, void *priv,
+               int components, bool update, bool dynamic, void *priv,
                void (*fill)(void *priv, float *data, int w, int h, int d));
 
 // Returns a GLSL-version appropriate "bvec"-like type. For GLSL 130+, this
 // returns bvecN. For GLSL 120, this returns vecN instead. The intended use of
 // this function is with mix(), which only accepts bvec in GLSL 130+.
 const char *sh_bvec(const struct pl_shader *sh, int dims);
+
+// Returns the appropriate `texture`-equivalent function for the shader and
+// given texture.
+static inline const char *sh_tex_fn(const struct pl_shader *sh,
+                                    const struct pl_tex_params params)
+{
+    static const char *suffixed[] = {
+        [1] = "texture1D",
+        [2] = "texture2D",
+        [3] = "texture3D",
+    };
+
+    int dims = pl_tex_params_dimension(params);
+    return sh_glsl(sh).version >= 130 ? "texture" : suffixed[dims];
+}
