@@ -33,7 +33,8 @@ static void shaderc_destroy(struct spirv_compiler *spirv)
     talloc_free(spirv);
 }
 
-static struct spirv_compiler *shaderc_create(struct pl_context *ctx)
+static struct spirv_compiler *shaderc_create(struct pl_context *ctx,
+                                             uint32_t api_version)
 {
     struct spirv_compiler *spirv = talloc_ptrtype_priv(NULL, spirv, struct priv);
     struct priv *p = TA_PRIV(spirv);
@@ -55,6 +56,10 @@ static struct spirv_compiler *shaderc_create(struct pl_context *ctx)
             shaderc_optimization_level_size);
 #endif
 
+    shaderc_compile_options_set_target_env(p->opts,
+            shaderc_target_env_vulkan,
+            api_version);
+
     int ver, rev;
     shaderc_get_spv_version(&ver, &rev);
     spirv->compiler_version = ver * 100 + rev;
@@ -100,13 +105,15 @@ static bool shaderc_compile(struct spirv_compiler *spirv, void *tactx,
 
     enum pl_log_level lev = errs ? PL_LOG_ERR : warn ? PL_LOG_INFO : PL_LOG_DEBUG;
 
+    int s = shaderc_result_get_compilation_status(res);
+    bool success = s == shaderc_compilation_status_success;
+    if (!success)
+        lev = PL_LOG_ERR;
+
     const char *msg = shaderc_result_get_error_message(res);
     if (msg[0])
         PL_MSG(spirv, lev, "shaderc output:\n%s", msg);
 
-    int s = shaderc_result_get_compilation_status(res);
-    bool success = s == shaderc_compilation_status_success;
-
     static const char *results[] = {
         [shaderc_compilation_status_success]            = "success",
         [shaderc_compilation_status_invalid_stage]      = "invalid stage",
