@@ -54,11 +54,17 @@ typedef void (*bench_fn)(struct pl_shader *sh, struct pl_shader_obj **state,
 
 static void run_bench(const struct pl_gpu *gpu, struct pl_dispatch *dp,
                       struct pl_shader_obj **state, const struct pl_tex *src,
-                      const struct pl_tex *fbo, bench_fn bench)
+                      const struct pl_tex *fbo, struct pl_timer *timer,
+                      bench_fn bench)
 {
     struct pl_shader *sh = pl_dispatch_begin(dp);
     bench(sh, state, src);
-    pl_dispatch_finish(dp, &sh, fbo, NULL, NULL);
+
+    pl_dispatch_finish(dp, &(struct pl_dispatch_params) {
+        .shader = &sh,
+        .target = fbo,
+        .timer = timer,
+    });
 }
 
 static void benchmark(const struct pl_gpu *gpu, const char *name, bench_fn bench)
@@ -86,7 +92,7 @@ static void benchmark(const struct pl_gpu *gpu, const char *name, bench_fn bench
     }
 
     // Run the benchmark and flush+block once to force shader compilation etc.
-    run_bench(gpu, dp, &state, src, fbos[0], bench);
+    run_bench(gpu, dp, &state, src, fbos[0], NULL, bench);
     pl_gpu_finish(gpu);
 
     // Perform the actual benchmark
@@ -94,25 +100,44 @@ static void benchmark(const struct pl_gpu *gpu, const char *name, bench_fn bench
     unsigned long frames = 0;
     int index = 0;
 
+    struct pl_timer *timer = pl_timer_create(gpu);
+    uint64_t gputime_total = 0;
+    unsigned long gputime_count = 0;
+    uint64_t gputime;
+
     gettimeofday(&start, NULL);
     do {
         frames++;
-        run_bench(gpu, dp, &state, src, fbos[index++], bench);
+        run_bench(gpu, dp, &state, src, fbos[index++], timer, bench);
         index %= NUM_FBOS;
-        if (index == 0)
+        if (index == 0) {
             pl_gpu_flush(gpu);
-        gettimeofday(&stop, NULL);
+            gettimeofday(&stop, NULL);
+        }
+        while ((gputime = pl_timer_query(gpu, timer))) {
+            gputime_total += gputime;
+            gputime_count++;
+        }
     } while (stop.tv_sec - start.tv_sec < BENCH_DUR);
 
     // Force the GPU to finish execution and re-measure the final stop time
     pl_gpu_finish(gpu);
+
     gettimeofday(&stop, NULL);
+    while ((gputime = pl_timer_query(gpu, timer))) {
+        gputime_total += gputime;
+        gputime_count++;
+    }
 
     float secs = (float) (stop.tv_sec - start.tv_sec) +
                  1e-6 * (stop.tv_usec - start.tv_usec);
-    printf("'%s':\t%4lu frames in %1.6f seconds => %2.6f ms/frame (%5.2f FPS)\n",
+    printf("'%s':\t%4lu frames in %1.6f seconds => %2.6f ms/frame (%5.2f FPS)",
           name, frames, secs, 1000 * secs / frames, frames / secs);
+    if (gputime_count)
+        printf(", gpu time: %2.6f ms", 1e-6 * (gputime_total / gputime_count));
+    printf("\n");
 
+    pl_timer_destroy(gpu, &timer);
     pl_shader_obj_destroy(&state);
     pl_dispatch_destroy(&dp);
     pl_tex_destroy(gpu, &src);
@@ -121,18 +146,6 @@ static void benchmark(const struct pl_gpu *gpu, const char *name, bench_fn bench
 }
 
 // List of benchmarks
-static void bench_bt2020c(struct pl_shader *sh, struct pl_shader_obj **state,
-                          const struct pl_tex *src)
-{
-    struct pl_color_repr repr = {
-        .sys    = PL_COLOR_SYSTEM_BT_2020_C,
-        .levels = PL_COLOR_LEVELS_TV,
-    };
-
-    pl_shader_sample_direct(sh, &(struct pl_sample_src) { .tex = src });
-    pl_shader_decode_color(sh, &repr, NULL);
-}
-
 static void bench_deband(struct pl_shader *sh, struct pl_shader_obj **state,
                          const struct pl_tex *src)
 {
@@ -183,17 +196,6 @@ static void bench_dither_white(struct pl_shader *sh, struct pl_shader_obj **stat
     pl_shader_dither(sh, 8, state, &params);
 }
 
-static void bench_dither_ordered_lut(struct pl_shader *sh,
-                                     struct pl_shader_obj **state,
-                                     const struct pl_tex *src)
-{
-    struct pl_dither_params params = pl_dither_default_params;
-    params.method = PL_DITHER_ORDERED_LUT;
-
-    pl_shader_sample_direct(sh, &(struct pl_sample_src) { .tex = src });
-    pl_shader_dither(sh, 8, state, &params);
-}
-
 static void bench_dither_ordered_fix(struct pl_shader *sh,
                                      struct pl_shader_obj **state,
                                      const struct pl_tex *src)
@@ -230,107 +232,42 @@ static void bench_polar_nocompute(struct pl_shader *sh,
 }
 
 
-static void bench_hdr_hable(struct pl_shader *sh, struct pl_shader_obj **state,
-                            const struct pl_tex *src)
-{
-    struct pl_color_map_params params = {
-        .tone_mapping_algo = PL_TONE_MAPPING_HABLE,
-    };
-
-    pl_shader_sample_direct(sh, &(struct pl_sample_src) { .tex = src });
-    pl_shader_color_map(sh, &params, pl_color_space_hdr10, pl_color_space_monitor,
-                        state, false);
-}
-
-static void bench_hdr_mobius(struct pl_shader *sh, struct pl_shader_obj **state,
-                             const struct pl_tex *src)
-{
-    struct pl_color_map_params params = {
-        .tone_mapping_algo = PL_TONE_MAPPING_MOBIUS,
-    };
-
-    pl_shader_sample_direct(sh, &(struct pl_sample_src) { .tex = src });
-    pl_shader_color_map(sh, &params, pl_color_space_hdr10, pl_color_space_monitor,
-                        state, false);
-}
-
 static void bench_hdr_peak(struct pl_shader *sh, struct pl_shader_obj **state,
                             const struct pl_tex *src)
 {
-    struct pl_color_map_params params = {
-        .tone_mapping_algo = PL_TONE_MAPPING_CLIP,
-        .peak_detect_frames = 10,
-    };
-
     pl_shader_sample_direct(sh, &(struct pl_sample_src) { .tex = src });
-    pl_shader_color_map(sh, &params, pl_color_space_hdr10, pl_color_space_monitor,
-                        state, false);
+    pl_shader_detect_peak(sh, pl_color_space_hdr10, state, NULL);
 }
 
-static void bench_hdr_desat(struct pl_shader *sh, struct pl_shader_obj **state,
-                            const struct pl_tex *src)
-{
-    struct pl_color_map_params params = {
-        .tone_mapping_algo = PL_TONE_MAPPING_CLIP,
-        .tone_mapping_desaturate = 1.0,
-    };
-
-    pl_shader_sample_direct(sh, &(struct pl_sample_src) { .tex = src });
-    pl_shader_color_map(sh, &params, pl_color_space_hdr10, pl_color_space_monitor,
-                        state, false);
-}
-
-static const struct pl_grain_params grain_params = {
-    .width = TEX_SIZE,
-    .height = TEX_SIZE,
-    .repr = {
-        .sys = PL_COLOR_SYSTEM_BT_709,
-        .levels = PL_COLOR_LEVELS_TV,
-        .bits = { .color_depth = 10, .sample_depth = 10 },
-    },
-    .grain_seed = 48476,
-
-    .num_points_y = 6,
-    .points_y = {{0, 4}, {27, 33}, {54, 55}, {67, 61}, {108, 71}, {255, 72}},
-    .chroma_scaling_from_luma = false,
-    .num_points_uv = {2, 2},
-    .points_uv = {{{0, 64}, {255, 64}}, {{0, 64}, {255, 64}}},
-    .scaling_shift = 11,
-    .ar_coeff_lag = 3,
-    .ar_coeffs_y = {4,   1, 3,   0,  1, -3,  8, -3,  7, -23, 1, -25,
-                    0, -10, 6, -17, -4, 53, 36,  5, -5, -17, 8,  66},
-    .ar_coeffs_uv = {
-        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 127},
-        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 127},
-    },
-    .ar_coeff_shift = 7,
-    .grain_scale_shift = 0,
-    .uv_mult = {0, 0},
-    .uv_mult_luma = {64, 64},
-    .uv_offset = {0, 0},
-};
-
 static void bench_av1_grain(struct pl_shader *sh, struct pl_shader_obj **state,
                             const struct pl_tex *src)
 {
-    struct pl_grain_params params = grain_params;
-    params.grain_seed = rand();
+    struct pl_av1_grain_params params = {
+        .data = av1_grain_data,
+        .tex = src,
+        .components = 3,
+        .component_mapping = {0, 1, 2},
+        .repr = &(struct pl_color_repr) {0},
+    };
 
-    pl_shader_sample_direct(sh, &(struct pl_sample_src) { .tex = src });
-    pl_shader_av1_grain(sh, state, (enum pl_channel[]){0, 1, 2}, NULL, &params);
+    params.data.grain_seed = rand();
+    pl_shader_av1_grain(sh, state, &params);
 }
 
 static void bench_av1_grain_lap(struct pl_shader *sh, struct pl_shader_obj **state,
                                 const struct pl_tex *src)
 {
-    struct pl_grain_params params = grain_params;
-    params.grain_seed = rand();
-    params.overlap = true;
+    struct pl_av1_grain_params params = {
+        .data = av1_grain_data,
+        .tex = src,
+        .components = 3,
+        .component_mapping = {0, 1, 2},
+        .repr = &(struct pl_color_repr) {0},
+    };
 
-    pl_shader_sample_direct(sh, &(struct pl_sample_src) { .tex = src });
-    pl_shader_av1_grain(sh, state, (enum pl_channel[]){0, 1, 2}, NULL, &params);
+    params.data.overlap = true;
+    params.data.grain_seed = rand();
+    pl_shader_av1_grain(sh, state, &params);
 }
 
 int main()
@@ -367,20 +304,17 @@ int main()
     // Dithering algorithms
     benchmark(vk->gpu, "dither_blue", bench_dither_blue);
     benchmark(vk->gpu, "dither_white", bench_dither_white);
-    benchmark(vk->gpu, "dither_ordered_lut", bench_dither_ordered_lut);
     benchmark(vk->gpu, "dither_ordered_fixed", bench_dither_ordered_fix);
 
-    // HDR tone mapping
-    benchmark(vk->gpu, "hdr_hable", bench_hdr_hable);
-    benchmark(vk->gpu, "hdr_mobius", bench_hdr_mobius);
-    benchmark(vk->gpu, "hdr_desaturate", bench_hdr_desat);
+    // HDR peak detection
     if (vk->gpu->caps & PL_GPU_CAP_COMPUTE)
         benchmark(vk->gpu, "hdr_peakdetect", bench_hdr_peak);
 
     // Misc stuff
-    benchmark(vk->gpu, "bt2020c", bench_bt2020c);
     benchmark(vk->gpu, "av1_grain", bench_av1_grain);
     benchmark(vk->gpu, "av1_grain_lap", bench_av1_grain_lap);
 
+    pl_vulkan_destroy(&vk);
+    pl_context_destroy(&ctx);
     return 0;
 }
