@@ -81,9 +81,14 @@ static void pl_test_roundtrip(const struct pl_gpu *gpu, const struct pl_tex *tex
     for (size_t i = 0; i < bytes; i++)
         src[i] = (RANDOM * 256);
 
+    struct pl_timer *ul, *dl;
+    ul = pl_timer_create(gpu);
+    dl = pl_timer_create(gpu);
+
     REQUIRE(pl_tex_upload(gpu, &(struct pl_tex_transfer_params){
         .tex = tex[0],
         .ptr = src,
+        .timer = ul,
     }));
 
     // Test blitting, if possible for this format
@@ -106,6 +111,7 @@ static void pl_test_roundtrip(const struct pl_gpu *gpu, const struct pl_tex *tex
     REQUIRE(pl_tex_download(gpu, &(struct pl_tex_transfer_params){
         .tex = dst_tex,
         .ptr = dst,
+        .timer = dl,
     }));
 
     if (fmt->emulated && fmt->type == PL_FMT_FLOAT) {
@@ -115,6 +121,14 @@ static void pl_test_roundtrip(const struct pl_gpu *gpu, const struct pl_tex *tex
     } else {
         REQUIRE(memcmp(src, dst, bytes) == 0);
     }
+
+    // Report timer results
+    pl_gpu_finish(gpu);
+    printf("upload time: %"PRIu64", download time: %"PRIu64"\n",
+           pl_timer_query(gpu, ul), pl_timer_query(gpu, dl));
+
+    pl_timer_destroy(gpu, &ul);
+    pl_timer_destroy(gpu, &dl);
 }
 
 static void pl_texture_tests(const struct pl_gpu *gpu)
@@ -260,15 +274,22 @@ static void pl_shader_tests(const struct pl_gpu *gpu)
     REQUIRE(pass);
     REQUIRE(pass->params.cached_program_len);
 
+    struct pl_timer *timer = pl_timer_create(gpu);
     pl_pass_run(gpu, &(struct pl_pass_run_params) {
         .pass           = pass,
         .target         = fbo,
         .vertex_data    = vertices,
         .vertex_count   = sizeof(vertices) / sizeof(struct vertex),
+        .timer          = timer,
     });
 
     pl_pass_destroy(gpu, &pass);
 
+    // Wait until this pass is complete and report the timer result
+    pl_gpu_finish(gpu);
+    printf("timer query result: %"PRIu64"\n", pl_timer_query(gpu, timer));
+    pl_timer_destroy(gpu, &timer);
+
     static float data[FBO_H * FBO_W * 4] = {0};
 
     // Test against the known pattern of `src`, only useful for roundtrip tests
@@ -283,8 +304,6 @@ static void pl_shader_tests(const struct pl_gpu *gpu)
         for (int y = 0; y < FBO_H; y++) {                                   \
             for (int x = 0; x < FBO_W; x++) {                               \
                 float *color = &data[(y * FBO_W + x) * 4];                  \
-                printf("color[%d][%d] = %f %f %f %f\n",                     \
-                       y, x, color[0], color[1], color[2], color[3]);       \
                 REQUIRE(feq(color[0], (x + 0.5) / FBO_W, eps));             \
                 REQUIRE(feq(color[1], (y + 0.5) / FBO_H, eps));             \
                 REQUIRE(feq(color[2], 0.0, eps));                           \
@@ -315,7 +334,10 @@ static void pl_shader_tests(const struct pl_gpu *gpu)
         pl_shader_sample_direct(sh, &(struct pl_sample_src) { .tex = src });
         pl_shader_delinearize(sh, trc);
         pl_shader_linearize(sh, trc);
-        REQUIRE(pl_dispatch_finish(dp, &sh, fbo, NULL, NULL));
+        REQUIRE(pl_dispatch_finish(dp, &(struct pl_dispatch_params) {
+            .shader = &sh,
+            .target = fbo,
+        }));
 
         float epsilon = pl_color_transfer_is_hdr(trc) ? 1e-4 : 1e-6;
         TEST_FBO_PATTERN(epsilon, "transfer function %d", (int) trc);
@@ -326,7 +348,10 @@ static void pl_shader_tests(const struct pl_gpu *gpu)
         pl_shader_sample_direct(sh, &(struct pl_sample_src) { .tex = src });
         pl_shader_encode_color(sh, &(struct pl_color_repr) { .sys = sys });
         pl_shader_decode_color(sh, &(struct pl_color_repr) { .sys = sys }, NULL);
-        REQUIRE(pl_dispatch_finish(dp, &sh, fbo, NULL, NULL));
+        REQUIRE(pl_dispatch_finish(dp, &(struct pl_dispatch_params) {
+            .shader = &sh,
+            .target = fbo,
+        }));
 
         float epsilon;
         switch (sys) {
@@ -348,6 +373,21 @@ static void pl_shader_tests(const struct pl_gpu *gpu)
         TEST_FBO_PATTERN(epsilon, "color system %d", (int) sys);
     }
 
+    for (enum pl_color_light light = 0; light < PL_COLOR_LIGHT_COUNT; light++) {
+        sh = pl_dispatch_begin(dp);
+        struct pl_color_space src_space = { .light = light };
+        struct pl_color_space dst_space = { 0 };
+        pl_shader_sample_direct(sh, &(struct pl_sample_src) { .tex = src });
+        pl_shader_color_map(sh, NULL, src_space, dst_space, NULL, false);
+        pl_shader_color_map(sh, NULL, dst_space, src_space, NULL, false);
+        REQUIRE(pl_dispatch_finish(dp, &(struct pl_dispatch_params) {
+            .shader = &sh,
+            .target = fbo,
+        }));
+
+        TEST_FBO_PATTERN(1e-6, "light %d", (int) light);
+    }
+
     // Repeat this a few times to test the caching
     for (int i = 0; i < 10; i++) {
         sh = pl_dispatch_begin(dp);
@@ -368,11 +408,14 @@ static void pl_shader_tests(const struct pl_gpu *gpu)
                 .grain          = 0.0,
         });
 
-        REQUIRE(pl_dispatch_finish(dp, &sh, fbo, NULL, NULL));
+        REQUIRE(pl_dispatch_finish(dp, &(struct pl_dispatch_params) {
+            .shader = &sh,
+            .target = fbo,
+        }));
         TEST_FBO_PATTERN(1e-6, "deband iter %d", i);
     }
 
-#if PL_HAVE_LCMS
+#ifdef PL_HAVE_LCMS
     // Test the use of 3DLUTs if available
     sh = pl_dispatch_begin(dp);
     pl_shader_sample_direct(sh, &(struct pl_sample_src) { .tex = src });
@@ -384,7 +427,10 @@ static void pl_shader_tests(const struct pl_gpu *gpu)
 
     if (pl_3dlut_update(sh, &src_color, &dst_color, &lut3d, &out, NULL)) {
         pl_3dlut_apply(sh, &lut3d);
-        REQUIRE(pl_dispatch_finish(dp, &sh, fbo, NULL, NULL));
+        REQUIRE(pl_dispatch_finish(dp, &(struct pl_dispatch_params) {
+            .shader = &sh,
+            .target = fbo,
+        }));
     }
 
     pl_dispatch_abort(dp, &sh);
@@ -396,9 +442,10 @@ static void pl_shader_tests(const struct pl_gpu *gpu)
     for (int i = 0; i < 2; i++) {
         struct pl_av1_grain_params grain_params = {
             .data = av1_grain_data,
-            .luma_tex = src,
-            .channels = { 0, 1, 2 },
-            .repr = {
+            .tex = src,
+            .components = 3,
+            .component_mapping = { 0, 1, 2 },
+            .repr = &(struct pl_color_repr) {
                 .sys = PL_COLOR_SYSTEM_BT_709,
                 .levels = PL_COLOR_LEVELS_TV,
                 .bits = { .color_depth = 10, .sample_depth = 10 },
@@ -408,9 +455,11 @@ static void pl_shader_tests(const struct pl_gpu *gpu)
         grain_params.data.overlap = !!i;
 
         sh = pl_dispatch_begin(dp);
-        pl_shader_sample_direct(sh, &(struct pl_sample_src) { .tex = src });
         pl_shader_av1_grain(sh, &grain, &grain_params);
-        REQUIRE(pl_dispatch_finish(dp, &sh, fbo, NULL, NULL));
+        REQUIRE(pl_dispatch_finish(dp, &(struct pl_dispatch_params) {
+            .shader = &sh,
+            .target = fbo,
+        }));
     }
     pl_shader_obj_destroy(&grain);
 
@@ -483,7 +532,10 @@ static void pl_scaler_tests(const struct pl_gpu *gpu)
             .no_compute = !fbo->params.storable,
         }
     ));
-    REQUIRE(pl_dispatch_finish(dp, &sh, fbo, NULL, NULL));
+    REQUIRE(pl_dispatch_finish(dp, &(struct pl_dispatch_params) {
+        .shader = &sh,
+        .target = fbo,
+    }));
 
     if (fbo->params.host_readable) {
         fbo_data = malloc(fbo->params.w * fbo->params.h * sizeof(float));
@@ -511,6 +563,55 @@ error:
     pl_tex_destroy(gpu, &fbo);
 }
 
+static const char *user_shader_tests[] = {
+
+    // Test hooking, saving and loading
+    "// Example of a comment at the beginning                               \n"
+    "                                                                       \n"
+    "//!HOOK NATIVE                                                         \n"
+    "//!DESC upscale image                                                  \n"
+    "//!BIND HOOKED                                                         \n"
+    "//!WIDTH HOOKED.w 10 *                                                 \n"
+    "//!HEIGHT HOOKED.h 10 *                                                \n"
+    "//!SAVE NATIVEBIG                                                      \n"
+    "//!WHEN NATIVE.w 500 <                                                 \n"
+    "                                                                       \n"
+    "vec4 hook()                                                            \n"
+    "{                                                                      \n"
+    "    return HOOKED_texOff(0);                                           \n"
+    "}                                                                      \n"
+    "                                                                       \n"
+    "//!HOOK MAIN                                                           \n"
+    "//!DESC downscale bigger image                                         \n"
+    "//!WHEN NATIVE.w 500 <                                                 \n"
+    "//!BIND NATIVEBIG                                                      \n"
+    "                                                                       \n"
+    "vec4 hook()                                                            \n"
+    "{                                                                      \n"
+    "    return NATIVEBIG_texOff(0);                                        \n"
+    "}                                                                      \n",
+
+    // Test use of textures
+    "//!HOOK MAIN                                                           \n"
+    "//!DESC turn everything into colorful pixels                           \n"
+    "//!BIND HOOKED                                                         \n"
+    "//!BIND DISCO                                                          \n"
+    "//!COMPONENTS 3                                                        \n"
+    "                                                                       \n"
+    "vec4 hook()                                                            \n"
+    "{                                                                      \n"
+    "    return vec4(DISCO_tex(HOOKED_pos * 10.0).rgb, 1);                  \n"
+    "}                                                                      \n"
+    "                                                                       \n"
+    "//!TEXTURE DISCO                                                       \n"
+    "//!SIZE 3 3                                                            \n"
+    "//!FORMAT rgba32f                                                      \n"
+    "//!FILTER NEAREST                                                      \n"
+    "//!BORDER REPEAT                                                       \n"
+    "0000803f000000000000000000000000000000000000803f000000000000000000000000000000000000803f00000000000000000000803f0000803f000000000000803f000000000000803f000000000000803f0000803f00000000000000009a99993e9a99993e9a99993e000000009a99193f9a99193f9a99193f000000000000803f0000803f0000803f00000000 \n",
+
+};
+
 static void pl_render_tests(const struct pl_gpu *gpu)
 {
     const struct pl_fmt *fbo_fmt = pl_find_fmt(gpu, PL_FMT_FLOAT, 4, 16, 32,
@@ -572,8 +673,6 @@ static void pl_render_tests(const struct pl_gpu *gpu)
             .levels     = PL_COLOR_LEVELS_PC,
         },
         .color          = pl_color_space_bt709,
-        .width          = width,
-        .height         = height,
         .src_rect       = {-1.0, 0.0, width - 1.0, height},
     };
 
@@ -634,7 +733,7 @@ static void pl_render_tests(const struct pl_gpu *gpu)
     // Test HDR stuff
     image.color.sig_scale = 10.0;
     target.color.sig_scale = 2.0;
-    TEST_PARAMS(color_map, tone_mapping_algo, PL_TONE_MAPPING_LINEAR);
+    TEST_PARAMS(color_map, tone_mapping_algo, PL_TONE_MAPPING_BT_2390);
     TEST_PARAMS(color_map, desaturation_strength, 1);
     image.color.sig_scale = target.color.sig_scale = 0.0;
 
@@ -657,6 +756,22 @@ static void pl_render_tests(const struct pl_gpu *gpu)
     REQUIRE(pl_render_image(rr, &image, &target, &params));
     image.av1_grain = (struct pl_av1_grain_data) {0};
 
+    // Test mpv-style custom shaders
+    for (int i = 0; i < PL_ARRAY_SIZE(user_shader_tests); i++) {
+        printf("testing user shader:\n\n%s\n", user_shader_tests[i]);
+        const struct pl_hook *hook;
+        hook = pl_mpv_user_shader_parse(gpu, user_shader_tests[i],
+                                        strlen(user_shader_tests[i]));
+        REQUIRE(hook);
+
+        params.hooks = &hook;
+        params.num_hooks = 1;
+        REQUIRE(pl_render_image(rr, &image, &target, &params));
+
+        pl_mpv_user_shader_destroy(&hook);
+    }
+    params = pl_render_default_params;
+
     // Test overlays
     image.num_overlays = 1;
     image.overlays = &(struct pl_overlay) {
