@@ -38,6 +38,12 @@ static void vulkan_interop_tests(const struct pl_vulkan *pl_vk,
 
         REQUIRE(sync);
         REQUIRE(tex);
+
+        // Note: For testing purposes, we have to fool pl_tex_export into
+        // thinking this texture is actually exportable. Just hack it in
+        // horribly.
+        ((struct pl_tex_params *) &tex->params)->export_handle = PL_HANDLE_DMA_BUF;
+
         REQUIRE(pl_tex_export(gpu, tex, sync));
 
         // Re-use our internal helpers to signal this VkSemaphore
@@ -47,7 +53,7 @@ static void vulkan_interop_tests(const struct pl_vulkan *pl_vk,
         REQUIRE(cmd);
         pl_vk_sync_unwrap(sync, NULL, &signal);
         vk_cmd_sig(cmd, signal);
-        vk_cmd_queue(vk, cmd);
+        vk_cmd_queue(vk, &cmd);
         REQUIRE(vk_flush_commands(vk));
 
         // Do something with the image again to "import" it
@@ -141,6 +147,7 @@ int main()
     const struct pl_vk_inst *inst;
     inst = pl_vk_inst_create(ctx, &(struct pl_vk_inst_params) {
         .debug = true,
+        .debug_extra = true,
         .opt_extensions = (const char *[]){
             VK_KHR_SURFACE_EXTENSION_NAME,
             "VK_EXT_headless_surface", // in case it isn't defined
@@ -206,13 +213,9 @@ int main()
         params.queue_count = 8; // test inter-queue stuff
         params.surface = surf;
 
-        if (props.vendorID == 0x8086 &&
-            props.deviceID == 0x3185 &&
-            props.driverVersion <= 79695878)
-        {
-            // Blacklist compute shaders for the CI's old intel iGPU..
-            params.blacklist_caps = PL_GPU_CAP_COMPUTE;
-        }
+#ifdef CI_BLACKLIST_COMPUTE
+        params.blacklist_caps = PL_GPU_CAP_COMPUTE;
+#endif
 
         const struct pl_vulkan *vk = pl_vulkan_create(ctx, &params);
         if (!vk)
@@ -221,6 +224,23 @@ int main()
         gpu_tests(vk->gpu);
         vulkan_swapchain_tests(vk, surf);
 
+        // Test importing this context via the vulkan interop API
+        struct pl_vulkan_import_params iparams = {
+            .instance = vk->instance,
+            .phys_device = vk->phys_device,
+            .device = vk->device,
+
+            .extensions = vk->extensions,
+            .num_extensions = vk->num_extensions,
+            .features = vk->features,
+            .queue_graphics = vk->queue_graphics,
+            .queue_compute = vk->queue_compute,
+            .queue_transfer = vk->queue_transfer,
+        };
+        const struct pl_vulkan *vk2 = pl_vulkan_import(ctx, &iparams);
+        REQUIRE(vk2);
+        pl_vulkan_destroy(&vk2);
+
         // Run these tests last because they disable some validation layers
 #ifdef VK_HAVE_UNIX
         vulkan_interop_tests(vk, PL_HANDLE_FD);
@@ -231,7 +251,29 @@ int main()
         vulkan_interop_tests(vk, PL_HANDLE_WIN32);
         vulkan_interop_tests(vk, PL_HANDLE_WIN32_KMT);
 #endif
+
         pl_vulkan_destroy(&vk);
+
+        // Re-run the same export/import tests with async queues disabled
+        params.async_compute = false;
+        params.async_transfer = false;
+        vk = pl_vulkan_create(ctx, &params);
+        REQUIRE(vk); // it succeeded the first time
+
+#ifdef VK_HAVE_UNIX
+        vulkan_interop_tests(vk, PL_HANDLE_FD);
+        vulkan_interop_tests(vk, PL_HANDLE_DMA_BUF);
+        vulkan_test_export_import(vk, PL_HANDLE_DMA_BUF);
+#endif
+#ifdef VK_HAVE_WIN32
+        vulkan_interop_tests(vk, PL_HANDLE_WIN32);
+        vulkan_interop_tests(vk, PL_HANDLE_WIN32_KMT);
+#endif
+
+        pl_vulkan_destroy(&vk);
+
+        // Reduce log spam after first tested device
+        pl_test_set_verbosity(ctx, PL_LOG_INFO);
     }
 
     vkDestroySurfaceKHR(inst->instance, surf, NULL);
