@@ -35,6 +35,7 @@ static void vk_cmd_reset(struct vk_ctx *vk, struct vk_cmd *cmd)
     cmd->num_callbacks = 0;
     cmd->num_deps = 0;
     cmd->num_sigs = 0;
+    cmd->num_objs = 0;
 
     // also make sure to reset vk->last_cmd in case this was the last command
     if (vk->last_cmd == cmd)
@@ -74,6 +75,7 @@ static struct vk_cmd *vk_cmd_create(struct vk_ctx *vk, struct vk_cmdpool *pool)
     };
 
     VK(vk->CreateFence(vk->dev, &finfo, VK_ALLOC, &cmd->fence));
+    VK_NAME(FENCE, cmd->fence, "cmd");
 
     return cmd;
 
@@ -113,6 +115,11 @@ void vk_cmd_dep(struct vk_cmd *cmd, VkSemaphore dep, VkPipelineStageFlags stage)
     cmd->depstages[idx] = stage;
 }
 
+void vk_cmd_obj(struct vk_cmd *cmd, const void *obj)
+{
+    TARRAY_APPEND(cmd, cmd->objs, cmd->num_objs, obj);
+}
+
 void vk_cmd_sig(struct vk_cmd *cmd, VkSemaphore sig)
 {
     TARRAY_APPEND(cmd, cmd->sigs, cmd->num_sigs, sig);
@@ -139,8 +146,10 @@ struct vk_signal *vk_cmd_signal(struct vk_ctx *vk, struct vk_cmd *cmd,
     };
 
     // We can skip creating the semaphores if there's only one queue
-    if (vk->num_pools > 1 || vk->pools[0]->num_queues > 1)
+    if (vk->num_pools > 1 || vk->pools[0]->num_queues > 1) {
         VK(vk->CreateSemaphore(vk->dev, &sinfo, VK_ALLOC, &sig->semaphore));
+        VK_NAME(SEMAPHORE, sig->semaphore, "sig");
+    }
 
     static const VkEventCreateInfo einfo = {
         .sType = VK_STRUCTURE_TYPE_EVENT_CREATE_INFO,
@@ -156,6 +165,7 @@ struct vk_signal *vk_cmd_signal(struct vk_ctx *vk, struct vk_cmd *cmd,
             PL_INFO(vk, "VkEvent creation failed.. disabling events");
         } else {
             VK_ASSERT(res, "Creating VkEvent");
+            VK_NAME(EVENT, sig->event, "sig");
         }
     }
 
@@ -313,8 +323,8 @@ void vk_cmdpool_destroy(struct vk_ctx *vk, struct vk_cmdpool *pool)
 
 struct vk_cmd *vk_cmd_begin(struct vk_ctx *vk, struct vk_cmdpool *pool)
 {
-    // garbage collect the cmdpool first, to increase the chances of getting
-    // an already-available command buffer
+    // Garbage collect the cmdpool first, to increase the chances of getting
+    // an already-available command buffer.
     vk_poll_commands(vk, 0);
 
     struct vk_cmd *cmd = NULL;
@@ -345,9 +355,11 @@ error:
     return NULL;
 }
 
-bool vk_cmd_queue(struct vk_ctx *vk, struct vk_cmd *cmd)
+bool vk_cmd_queue(struct vk_ctx *vk, struct vk_cmd **pcmd)
 {
+    struct vk_cmd *cmd = *pcmd;
     struct vk_cmdpool *pool = cmd->pool;
+    *pcmd = NULL;
 
     VK(vk->EndCommandBuffer(cmd->buf));
 
@@ -374,9 +386,6 @@ bool vk_poll_commands(struct vk_ctx *vk, uint64_t timeout)
 {
     bool ret = false;
 
-    if (timeout)
-        vk_flush_commands(vk);
-
     while (vk->num_cmds_pending > 0) {
         struct vk_cmd *cmd = vk->cmds_pending[0];
         struct vk_cmdpool *pool = cmd->pool;
@@ -402,9 +411,37 @@ bool vk_poll_commands(struct vk_ctx *vk, uint64_t timeout)
 
 bool vk_flush_commands(struct vk_ctx *vk)
 {
+    return vk_flush_obj(vk, NULL);
+}
+
+bool vk_flush_obj(struct vk_ctx *vk, const void *obj)
+{
+    // Count how many commands we want to flush
+    int num_to_flush = vk->num_cmds_queued;
+    if (obj) {
+        num_to_flush = 0;
+        for (int i = 0; i < vk->num_cmds_queued; i++) {
+            struct vk_cmd *cmd = vk->cmds_queued[i];
+            for (int o = 0; o < cmd->num_objs; o++) {
+                if (cmd->objs[o] == obj) {
+                    num_to_flush = i+1;
+                    goto next_cmd;
+                }
+            }
+
+next_cmd: ;
+        }
+    }
+
+    if (!num_to_flush)
+        return true;
+
+    PL_TRACE(vk, "Flushing %d/%d queued commands",
+             num_to_flush, vk->num_cmds_queued);
+
     bool ret = true;
 
-    for (int i = 0; i < vk->num_cmds_queued; i++) {
+    for (int i = 0; i < num_to_flush; i++) {
         struct vk_cmd *cmd = vk->cmds_queued[i];
         struct vk_cmdpool *pool = cmd->pool;
 
@@ -422,11 +459,15 @@ bool vk_flush_commands(struct vk_ctx *vk)
         if (pl_msg_test(vk->ctx, PL_LOG_TRACE)) {
             PL_TRACE(vk, "Submitting command on queue %p (QF %d):",
                      (void *)cmd->queue, pool->qf);
+            for (int n = 0; n < cmd->num_objs; n++)
+                PL_TRACE(vk, "    uses object %p", cmd->objs[n]);
             for (int n = 0; n < cmd->num_deps; n++)
                 PL_TRACE(vk, "    waits on semaphore %p", (void *) cmd->deps[n]);
             for (int n = 0; n < cmd->num_sigs; n++)
                 PL_TRACE(vk, "    signals semaphore %p", (void *) cmd->sigs[n]);
             PL_TRACE(vk, "    signals fence %p", (void *) cmd->fence);
+            if (cmd->num_callbacks)
+                PL_TRACE(vk, "    signals %d callbacks", cmd->num_callbacks);
         }
 
         VK(vk->QueueSubmit(cmd->queue, 1, &sinfo, cmd->fence));
@@ -440,7 +481,12 @@ error:
         ret = false;
     }
 
-    vk->num_cmds_queued = 0;
+    // Move remaining commands back to index 0
+    vk->num_cmds_queued -= num_to_flush;
+    if (vk->num_cmds_queued) {
+        memmove(vk->cmds_queued, &vk->cmds_queued[num_to_flush],
+                vk->num_cmds_queued * sizeof(vk->cmds_queued[0]));
+    }
 
     // Wait until we've processed some of the now pending commands
     while (vk->num_cmds_pending > PL_VK_MAX_PENDING_CMDS)
@@ -461,6 +507,6 @@ void vk_rotate_queues(struct vk_ctx *vk)
 
 void vk_wait_idle(struct vk_ctx *vk)
 {
-    // This also implicitly flushes the commands
+    vk_flush_commands(vk);
     while (vk_poll_commands(vk, UINT64_MAX)) ;
 }
