@@ -24,10 +24,6 @@
 
 const struct pl_gpu *pl_gpu_create_vk(struct vk_ctx *vk);
 
-// May be called on a struct ra of any type. Returns NULL if the ra is not
-// a vulkan ra.
-struct vk_ctx *pl_vk_get(const struct pl_gpu *gpu);
-
 // Associates an external semaphore (dependency) with a pl_tex, such that this
 // pl_tex will not be used by the pl_vk until the external semaphore fires.
 void pl_tex_vk_external_dep(const struct pl_gpu *gpu, const struct pl_tex *tex,
