@@ -38,8 +38,8 @@
 // Controls the maximum slab size, to reduce the effect of unbounded slab
 // growth exhausting memory. If the application needs a single allocation
 // that's bigger than this value, it will be allocated directly from the
-// device. (Default: 512 MB)
-#define PLVK_HEAP_MAXIMUM_SLAB_SIZE (1 << 29)
+// device. (Default: 256 MB)
+#define PLVK_HEAP_MAXIMUM_SLAB_SIZE (1 << 28)
 
 // Controls the minimum free region size, to reduce thrashing the free space
 // map with lots of small buffers during uninit. (Default: 1 KB)
@@ -260,6 +260,7 @@ static struct vk_slab *slab_alloc(struct vk_malloc *ma, struct vk_heap *heap,
         }
 
         VK(vk->CreateBuffer(vk->dev, &binfo, VK_ALLOC, &slab->buffer));
+        VK_NAME(BUFFER, slab->buffer, "slab");
 
         VkMemoryRequirements reqs = {0};
         vk->GetBufferMemoryRequirements(vk->dev, slab->buffer, &reqs);
@@ -513,7 +514,8 @@ static bool heap_get_region(struct vk_malloc *ma, struct vk_heap *heap,
     // with the heap
     if (size > PLVK_HEAP_MAXIMUM_SLAB_SIZE) {
         slab = slab_alloc(ma, heap, size);
-        slab->dedicated = true;
+        if (slab)
+            slab->dedicated = true;
         *out_slab = slab;
         *out_index = 0;
         return !!slab;
